@@ -9,6 +9,7 @@ import inspect
 import logging
 import os
 import pathlib
+import subprocess
 import sys
 import typing
 from collections.abc import Iterator
@@ -178,14 +179,24 @@ class Context:
         capture: bool = False,
         interactive: bool = False,
         **kwargs,
-    ) -> CompletedProcess[str]:
+    ) -> CompletedProcess[str] | None:
         """
         Run a subprocess.
 
         Either in a virtualenv context if one was configured or the system context.
         """
-        if self.venv:
-            return self.venv.run(
+        try:
+            if self.venv:
+                return self.venv.run(
+                    *cmdline,
+                    check=check,
+                    timeout_secs=timeout_secs,
+                    no_output_timeout_secs=no_output_timeout_secs,
+                    capture=capture,
+                    interactive=interactive,
+                    **kwargs,
+                )
+            return self._run(
                 *cmdline,
                 check=check,
                 timeout_secs=timeout_secs,
@@ -194,15 +205,10 @@ class Context:
                 interactive=interactive,
                 **kwargs,
             )
-        return self._run(
-            *cmdline,
-            check=check,
-            timeout_secs=timeout_secs,
-            no_output_timeout_secs=no_output_timeout_secs,
-            capture=capture,
-            interactive=interactive,
-            **kwargs,
-        )
+        except subprocess.CalledProcessError as exc:
+            self.error(str(exc))
+            self.exit(exc.returncode)
+        return None
 
     @contextmanager
     def chdir(self, path: pathlib.Path) -> Iterator[pathlib.Path]:
@@ -407,6 +413,43 @@ class Parser:
         return getattr(self.parser, attr)
 
 
+class GroupReference:
+    """
+    Simple class to hold tools command group names comparable to how
+    they would be invoked using the CLI.
+
+    For example, tools vm create is stored as ("tools", "vm", "create")
+    """
+
+    _instance: GroupReference | None = None
+    _commands: list[str]
+
+    def __new__(cls):
+        """
+        Method that instantiates a singleton class and returns it.
+        """
+        if cls._instance is None:
+            instance = super().__new__(cls)
+            instance._commands = {}
+            cls._instance = instance
+        return cls._instance
+
+    @classmethod
+    def add_command(cls, cli_name: tuple[str], group: CommandGroup) -> None:
+        """
+        Add a tools command.
+        """
+        instance = cls()
+        if cli_name not in instance._commands:
+            instance._commands[cli_name] = group
+
+    def __getitem__(self, item):
+        """
+        Propogate getting a command parser to the underlying dict.
+        """
+        return self._commands[item]
+
+
 class CommandGroup:
     """
     Command group which holds the available tool functions.
@@ -418,6 +461,16 @@ class CommandGroup:
             description = help
         if parent is None:
             parent = Parser()
+            GroupReference.add_command((name,), self)
+        # We can also pass a string or list of strings that specify the parent commands.
+        # This should help avoid circular imports
+        if isinstance(parent, str):
+            parent = [parent]
+        if isinstance(parent, list):
+            # NOTE: This means ordering of imports is important, but better than risking circular imports
+            GroupReference.add_command(tuple(parent + [name]), self)
+            parent = GroupReference()[tuple(parent)]
+
         self.venv_config = venv_config or {}
         self.parser = parent.subparsers.add_parser(
             name.replace("_", "-"),
